/***********************************************************************

 GetVersion:  Display and copy to clipboard results of
              GetVersion, GetVersionExA, and GetVersionExW.

 GetVersion.c
 jumper, 2019

 29Jan2019 start
 01Mar2019 done

\***********************************************************************/

#define szAppName "GetVersion"
#define hwndMain NULL


/****** Standard #define and #include statements ******/

#define WIN32_LEAN_AND_MEAN
#define STRICT
#include <windows.h>


/****** Functions ******/

int _cdecl mbf (LPCSTR psz, ...) {
  char      sz1k[1024];
  wvsprintf(sz1k, psz, (LPSTR)(&psz + 1));
  return    MessageBox (hwndMain, sz1k, szAppName, MB_ICONINFORMATION);
}


PROC GetKernel32ProcAddress (PSTR pProc) {
  return GetProcAddress (LoadLibrary ("Kernel32"), pProc);
}


BOOL CopyText(LPSTR sz)
{
    HGLOBAL hglbCopy;

    // Open the clipboard, and empty it.
    if (!OpenClipboard(NULL)) return FALSE;
    EmptyClipboard();

    // Allocate a global memory object for the DIB.
    hglbCopy = GlobalAlloc(GMEM_DDESHARE, 4096);
    if (hglbCopy == NULL) {
        CloseClipboard();
        return FALSE;
    }

    // Lock the handle and copy the text to the buffer.
    lstrcpy (GlobalLock(hglbCopy), sz);
    GlobalUnlock  (hglbCopy);

    // Place the handle on the clipboard.
    SetClipboardData (CF_OEMTEXT, hglbCopy);

    // Close the clipboard.
    CloseClipboard();
    return TRUE;
}


void WinMainCRTStartup (void) {
  static  OSVERSIONINFOA    via;
  static  OSVERSIONINFOEXA  viexa;
  static  OSVERSIONINFOW    viw;
  static  OSVERSIONINFOEXW  viexw;
  PROC    proc;
  char    sz2k[2048], *pTOP= sz2k+4;
  BOOL    bCopyText;

  /* GetVersion() */
  int   c = wsprintf (sz2k, "<tt>GetVersion:\t\t0x%08X", GetVersion ());

  /* GetVersionExA (OSVERSIONINFOA*) */
  via.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);
  c += wsprintf (sz2k+c, "\r\n\r\n"
    "GetVersionExA (OSVERSIONINFOA*):\r\n"
    "   dwOSVersionInfoSize\t0x%08X\r\n"
    "   dwMajorVersion     \t0x%08X\r\n"
    "   dwMinorVersion     \t0x%08X\r\n"
    "   dwBuildNumber      \t0x%08X\r\n"
    "   dwPlatformId\t\t0x%08X\r\n"
    "   szCSDVersion[128]\t\"%s\" [%02X %02X %02X %02X ...]\r\n"
    "   return %s",
    via.dwOSVersionInfoSize,
    via.dwMajorVersion,
    via.dwMinorVersion,
    via.dwBuildNumber,
    via.dwPlatformId,
    via.szCSDVersion,
    via.szCSDVersion[0], via.szCSDVersion[1],
    via.szCSDVersion[2], via.szCSDVersion[3],
    GetVersionExA(&via)? "TRUE": "FALSE"
  );

  /* GetVersionExA (OSVERSIONINFOEXA*) */
  viexa.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXA);
  if (GetVersionExA ((POSVERSIONINFOA)&viexa)) {  // cast needed in VC5
    c += wsprintf (sz2k+c, "\r\n\r\n"
      "GetVersionExA (OSVERSIONINFOEXA*):\r\n"
      "   dwOSVersionInfoSize\t0x%08X\r\n"
      "   dwMajorVersion     \t0x%08X\r\n"
      "   dwMinorVersion     \t0x%08X\r\n"
      "   dwBuildNumber      \t0x%08X\r\n"
      "   dwPlatformId\t\t0x%08X\r\n"
      "   szCSDVersion[128]\t\"%s\" [%02X %02X %02X %02X ...]\r\n"
      "   wServicePackMajor\t0x%04X\r\n"
      "   wServicePackMinor\t0x%04X\r\n"
      "   wSuiteMask\t\t0x%04X\r\n"
      "   wProductType\t\t0x%02X\r\n"
      "   wReserved\t\t0x%02X\r\n"
      "   return TRUE",
      viexa.dwOSVersionInfoSize,
      viexa.dwMajorVersion,
      viexa.dwMinorVersion,
      viexa.dwBuildNumber,
      viexa.dwPlatformId,
      viexa.szCSDVersion,
      viexa.szCSDVersion[0], viexa.szCSDVersion[1],
      viexa.szCSDVersion[2], viexa.szCSDVersion[3],
      viexa.wServicePackMajor,
      viexa.wServicePackMinor,
      viexa.wSuiteMask,
      viexa.wProductType,
      viexa.wReserved
    );
  } else {
    c += wsprintf (sz2k+c, "\r\n\r\n"
      "GetVersionExA (OSVERSIONINFOEXA*):\r\n"
      "   dwOSVersionInfoSize\t0x%08X\r\n"
      "   return FALSE",
      viexa.dwOSVersionInfoSize
    );
  }

  /* Check for GetVersionExW */
  proc = GetKernel32ProcAddress("GetVersionExW");
  if (!proc) {
    c += wsprintf (sz2k+c, "\r\n\r\nGetVersionExW:\r\n   <missing>");

  } else if (IDOK ==
             MessageBox (NULL, sz2k+4, szAppName " - page 1 of 2",
                         MB_ICONINFORMATION+MB_OKCANCEL)) {
    PBYTE pb = (PBYTE)viw.szCSDVersion;
    pTOP = sz2k + c + 4;

    /* GetVersionExW (OSVERSIONINFOW*) */
    viw.dwOSVersionInfoSize = sizeof(OSVERSIONINFOW);
    if (proc (&viw)) {
      c += wsprintf (sz2k+c, "\r\n\r\n"
        "GetVersionExW (OSVERSIONINFOW*):\r\n"
        "   dwOSVersionInfoSize\t0x%08X\r\n"
        "   dwMajorVersion     \t0x%08X\r\n"
        "   dwMinorVersion     \t0x%08X\r\n"
        "   dwBuildNumber      \t0x%08X\r\n"
        "   dwPlatformId\t\t0x%08X\r\n"
        "   szCSDVersion[128]\t\"%ws\" [%02X %02X %02X %02X ...]\r\n"
        "   return TRUE",
        viw.dwOSVersionInfoSize,
        viw.dwMajorVersion,
        viw.dwMinorVersion,
        viw.dwBuildNumber,
        viw.dwPlatformId,
        viw.szCSDVersion,
        pb[0], pb[1], pb[2], pb[3]
      );
    } else {
      c += wsprintf (sz2k+c, "\r\n\r\n"
        "GetVersionExW (OSVERSIONINFOW*):\r\n"
        "   dwOSVersionInfoSize\t0x%08X\r\n"
        "   return FALSE",
        viw.dwOSVersionInfoSize
      );
    }

    /* GetVersionExW (OSVERSIONINFOEXW*) */
    pb = (PBYTE)viexw.szCSDVersion;
    viexw.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXW);
    if (proc (&viexw)) {
      c += wsprintf (sz2k+c, "\r\n\r\n"
        "GetVersionExW (OSVERSIONINFOEXW*):\r\n"
        "   dwOSVersionInfoSize\t0x%08X\r\n"
        "   dwMajorVersion     \t0x%08X\r\n"
        "   dwMinorVersion     \t0x%08X\r\n"
        "   dwBuildNumber      \t0x%08X\r\n"
        "   dwPlatformId\t\t0x%08X\r\n"
        "   szCSDVersion[128]\t\"%ws\" [%02X %02X %02X %02X ...]\r\n"
        "   wServicePackMajor\t0x%04X\r\n"
        "   wServicePackMinor\t0x%04X\r\n"
        "   wSuiteMask\t\t0x%04X\r\n"
        "   wProductType\t\t0x%02X\r\n"
        "   wReserved\t\t0x%02X\r\n"
        "   return TRUE",
        viexw.dwOSVersionInfoSize,
        viexw.dwMajorVersion,
        viexw.dwMinorVersion,
        viexw.dwBuildNumber,
        viexw.dwPlatformId,
        viexw.szCSDVersion,
        pb[0], pb[1], pb[2], pb[3],
        viexw.wServicePackMajor,
        viexw.wServicePackMinor,
        viexw.wSuiteMask,
        viexw.wProductType,
        viexw.wReserved
      );
    } else {
      c += wsprintf (sz2k+c, "\r\n\r\n"
        "GetVersionExW (OSVERSIONINFOEXW*):\r\n"
        "   dwOSVersionInfoSize\t0x%08X\r\n"
        "   return FALSE",
        viexw.dwOSVersionInfoSize
      );
    }

  } else {  // Cancel
    ExitProcess (0);

  } // if !proc elseif Okay

  c += wsprintf (sz2k+c, "\r\n\r\n"
    "gv=0x%08X\r\n"
    "gvex=%X,%X,%X,%X,%s,%X,%X,%X,%X,%X</tt>",
    GetVersion (),
    via.dwMajorVersion,
    via.dwMinorVersion,
    via.dwBuildNumber,
    via.dwPlatformId,
    via.szCSDVersion,
    viexa.wServicePackMajor,
    viexa.wServicePackMinor,
    viexa.wSuiteMask,
    viexa.wProductType,
    viexa.wReserved
  );

  bCopyText = CopyText (sz2k);
  sz2k[c-5] = 0;  //trim "</tt>" from message before displaying
  MessageBox (hwndMain,
              pTOP,
              bCopyText? szAppName " - results on clipboard": szAppName,
              MB_ICONINFORMATION);

  ExitProcess (0);
}
